package com.ningdatech.pmapi.meeting.helper;

import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.pmapi.meeting.entity.domain.Meeting;
import com.ningdatech.pmapi.meeting.entity.domain.MeetingExpert;
import com.ningdatech.pmapi.sms.constant.VoiceSmsTemplateConst;
import com.ningdatech.pmapi.sms.utils.DateUtil;
import com.ningdatech.yxt.client.YxtClient;
import com.ningdatech.yxt.constants.YxtSmsSignEnum;
import com.ningdatech.yxt.model.cmd.SendSmsCmd;
import com.ningdatech.yxt.model.cmd.SendSmsCmd.SendSmsContext;
import com.ningdatech.yxt.model.cmd.SubmitTaskCallResponse;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;

import static com.ningdatech.yxt.model.cmd.SubmitTaskCallCmd.SubmitTaskCallContext;
import static com.ningdatech.yxt.model.cmd.SubmitTaskCallCmd.of;

/**
 * <p>
 * YxtCallOrSmsHelper
 * </p>
 *
 * @author WendyYang
 * @since 23:59 2022/8/31
 */
@Component
@AllArgsConstructor
public class YxtCallOrSmsHelper {

    private final YxtClient yxtClient;

    public void callByMeetingExperts(Meeting meeting, List<MeetingExpert> experts) {
        String callContent = String.format(VoiceSmsTemplateConst.OFFLINE_TEMPLATE,
                meeting.getHoldOrg(), meeting.getName(),
                meeting.getStartTime().format(DateUtil.DTF_YMD_HM), meeting.getMeetingAddress());
        List<SubmitTaskCallContext> callContexts = CollUtils.convert(experts, w -> {
            SubmitTaskCallContext context = new SubmitTaskCallContext();
            context.setContent(callContent);
            context.setReceiveNumber(w.getMobile());
            return context;
        });
        SubmitTaskCallResponse callResponse = yxtClient.submitTaskCall(of(callContexts));
        experts.forEach(w -> w.setSubmitKey(callResponse.getSubmitKey()));
    }

    public void sendSms(List<SendSmsContext> smsList) {
        yxtClient.submitSmsTask(new SendSmsCmd() {{
            setContextList(smsList);
            setSmsSignEnum(YxtSmsSignEnum.ZJS_ELECTRONIC_EXPERT_LIB);
        }});
    }

    public void sendSms(SendSmsContext sms) {
        sendSms(Collections.singletonList(sms));
    }

}
