package com.ningdatech.pmapi.projectdeclared.utils;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.util.StrPool;
import com.ningdatech.pmapi.fiscal.entity.CompanyFiscalCode;
import com.ningdatech.pmapi.fiscal.service.ICompanyFiscalCodeService;
import com.ningdatech.pmapi.projectdeclared.contants.DeclaredProjectContant;
import com.ningdatech.pmapi.projectdeclared.model.dto.DefaultDeclaredDTO;
import com.ningdatech.pmapi.projectlib.model.dto.ProjectDTO;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Objects;

/**
 * 生成项目编号工具类
 *
 * @author CMM
 * @since 2023/04/04 14:13
 */
@Component
public class GenerateProjectCodeUtil {
    @Autowired
    private ICompanyFiscalCodeService companyFiscalCodeService;

    @Autowired
    private IProjectService projectService;

    public String generateProjectCode(ProjectDTO projectInfo){
        // 获取所属行政区划代码(6位)
        String areaCode = projectInfo.getAreaCode();
        // 获取建设年度后两位
        Integer projectYear = projectInfo.getProjectYear();
        String year = String.valueOf(projectYear)
                .substring(DeclaredProjectContant.Project.YEAR_THREE, DeclaredProjectContant.Project.YEAR_FOUR);
        // 9-12位固定
        String fixedNumber = DeclaredProjectContant.Project.FIXED_NUMBER;
        // 获取公司的财政编码
        CompanyFiscalCode companyFiscalCode = companyFiscalCodeService.getByOrganizationCode(projectInfo.getBuildOrgCode());

        // 从表中查出单位配置的财政编码
        if (Objects.isNull(companyFiscalCode)){
            throw new BizException("申报单位未配置财政编码!");
        }

        String fiscalCode = companyFiscalCode.getFiscalCode();
        // 获取项目编号
        String projectIdStr;
        // 获取项目库当前最大项目序号
        List<Project> projectList = projectService.list(Wrappers.lambdaQuery(Project.class).orderByDesc(Project::getId));
        Project project = projectList.get(0);
        Long projectId = project.getId();
        if (projectId > DeclaredProjectContant.Project.PROJECT_ID){
            // 超过999的项目号从1开始 1
            Long newProjectId = projectId - DeclaredProjectContant.Project.PROJECT_ID;
            projectIdStr = String.format(StrPool.FORMAT_NUMBER_THREE, newProjectId);
        }else {
            projectIdStr = String.format(StrPool.FORMAT_NUMBER_THREE, projectId);
        }
        // 生成21位的项目编号
        return areaCode + year + fixedNumber + fiscalCode + projectIdStr;
    }
}
